warn "[DEPRECATION] This gem has been renamed to 'omniauth-entra-id' and will no longer be supported. Please switch to 'omniauth-entra-id' as soon as possible."

require File.join('omniauth', 'azure_activedirectory_v2', 'version')
require File.join('omniauth', 'strategies', 'azure_activedirectory_v2')
